@@ -7,9 +7,7 @@ use LWP;
 use JSON;
 use String::Random;
 
-use Data::Dumper;
-
-our $VERSION = "1.05";
+our $VERSION = "1.06";
 
 has 'server' => (
     is       => 'rw',
@@ -23,20 +21,31 @@ has 'password' => (
     is       => 'rw',
     required => 1,
 );
-has 'auth'				=> ( is => 'ro', );
-has 'ua'				=> ( is => 'ro', );
-has 'post_response'		=> ( is => 'ro');
-has 'last_response'		=> ( is => 'ro', );
-has 'json_request'		=> ( is => 'ro');
-has 'json_response'		=> ( is => 'ro');
-has 'verify_hostname'	=> ( is => 'rw', default => sub {1} );
-has 'ssl_opts'			=> ( is => 'rw');
-
+has 'zabbix_method'   => ( is => 'ro' );
+has 'zabbix_params'   => ( is => 'ro' );
+has 'auth'            => ( is => 'ro' );
+has 'ua'              => ( 
+    is      => 'ro',
+    #isa     => 'LWP::UserAgent',
+    lazy    => 1,
+    default => sub { LWP::UserAgent->new },
+);
+has 'post_response'   => ( is => 'ro' );
+has 'last_response'   => ( is => 'ro' );
+has 'json_request'    => ( is => 'ro' );
+has 'json_response'   => ( is => 'ro' );
+has 'verify_hostname' => ( is => 'rw', default => sub { 1 } );
+has 'ssl_opts'        => ( is => 'rw' );
+has 'delay'           => ( is => 'rw' );
+has 'request'         => ( is => 'ro' );
+has 'json_prepared'   => ( is => 'ro' );
+has 'json_executed'   => ( is => 'ro', default => sub { 0 } );
+has 'redo'            => ( is => 'ro' );
 my @content_type = ( 'content-type', 'application/json', );
 
 sub BUILD {
     my $self = shift;
-    $self->{ua} = LWP::UserAgent->new;
+    #$self->{ua} = LWP::UserAgent->new;
     my $ua        = $self->ua;
     my $url       = $self->server;
     my $id        = new String::Random;
@@ -49,66 +58,114 @@ sub BUILD {
             password => $self->password,
         },
     };
-	if ($self->verify_hostname == 0) {
-		$ua->ssl_opts(verify_hostname => 0);
-	}
-	
-	if ($self->ssl_opts) {
-		$ua->ssl_opts(%{ $self->{ssl_opts} });
-	}
+    if ( $self->verify_hostname == 0 ) {
+        $ua->ssl_opts( verify_hostname => 0 );
+    }
 
-    my $json = encode_json($json_data);
-    $self->{post_response} = $ua->post( $url, @content_type, Content => $json );
-	if ($self->{post_response}->{_rc} !~ /2\d\d/) {
-		die("$self->{post_response}->{_msg}");
-	}
-	$self->{json_request} = $self->{post_response}->{'_request'}->{_content};
-	$self->{json_response} = $self->{post_response}->{_content};
-    $self->{last_response} =
-      decode_json( $self->{post_response}->{_content} ) or die ($!);
-    if ( $self->{last_response}->{error} ) {
-        my $error = $self->{last_response}->{error}->{data};
-        croak("Error: $error");
+    if ( $self->ssl_opts ) {
+        $ua->ssl_opts( %{ $self->{ssl_opts} } );
     }
-    $self->{auth} = $self->{last_response}->{'result'};
 }
 
-sub do {
+sub login {
     my $self      = shift;
-    my $method    = shift;
-    my @args      = @_;
     my $id        = new String::Random;
     my $ua        = $self->ua;
-    my $auth      = $self->auth;
     my $url       = $self->server;
-    my $params;
-    if (scalar @args == 1) {
-        $params = $args[0];
-    }
-    else {
-        my %params = @args;
-        $params = \%params;
-    }
-
     my $json_data = {
         jsonrpc => '2.0',
         id      => $id->randpattern("nnnnnnnnnn"),
-        method  => $method,
-        auth    => $auth,
-        params  => $params,
+        method  => 'user.login',
+        params  => {
+            user     => $self->user,
+            password => $self->password,
+        },
     };
-    my $json = encode_json($json_data) or die($!);
-    $self->{post_response} = $ua->post( $url, @content_type, Content => $json );
-	$self->{json_request} = $self->{post_response}->{'_request'}->{_content};
-	$self->{json_response} = $self->{post_response}->{_content};
-    $self->{last_response} =
-      decode_json( $self->{post_response}->{_content} );
+    my $json = encode_json($json_data);
+    my $response = $ua->post( $url, @content_type, Content => $json );
+    if ( $response->{_rc} !~ /2\d\d/ ) {
+        die("$response->{_msg}");
+    }
+    my $content = decode_json( $response->{_content} ) or die($!);
+    if ( $content->{error} ) {
+        my $error_data = $content->{error}->{data};
+        my $error_msg  = $content->{error}->{message};
+        my $error_code = $content->{error}->{code};
+        my $error = "Error from Zabbix (code $error_code): $error_msg  $error_data";
+        croak($error);
+    }
+    $self->{auth} = $content->{'result'};
+}
 
+sub prepare {
+    my $self = shift;
+    login($self) if ( !$self->auth );
+    my $id     = new String::Random;
+    my $method = shift;
+    if ($method) {
+        $self->{zabbix_method} = $method;
+        my @args = @_;
+        if ( scalar @args == 1 ) {
+            $self->{zabbix_params} = $args[0];
+        }
+        else {
+            my %params = @args;
+            $self->{zabbix_params} = \%params;
+        }
+    }
+    if ( !$self->zabbix_method ) {
+        croak("No Zabbix API method defined");
+    }
+    $self->{request} = {
+        jsonrpc => '2.0',
+        id      => $id->randpattern("nnnnnnnnnn"),
+        method  => $self->zabbix_method,
+        auth    => $self->auth,
+        params  => $self->zabbix_params,
+    };
+    $self->{json_prepared} = encode_json( $self->request ) or die($!);
+}
+
+sub execute {
+    my $self = shift;
+    my $ua   = $self->ua;
+    $self->{post_response} = $ua->post( $self->server, @content_type,
+        Content => $self->json_prepared );
+    $self->{json_request}  = $self->{post_response}->{'_request'}->{_content};
+    $self->{json_response} = $self->{post_response}->{_content};
+    $self->{last_response} = decode_json( $self->{post_response}->{_content} );
+    my $method = $self->zabbix_method;
+    my $params = $self->zabbix_params;
+    prepare($self); ## Rerun prepare to get the new request id.
+}
+
+sub do {
+    my $self   = shift;
+    my $method = shift;
+    my @args   = @_;
+    if ($method) {
+        prepare( $self, $method, @args );
+    }
+    execute($self);
     if ( $self->{last_response}->{error} ) {
         my $error = $self->{last_response}->{error}->{data};
-        croak("Error: $error");
+        if (   ( !$self->{redo} )
+            && ( $error eq 'Session terminated, re-login, please.' ) )
+        {
+            $self->{redo}++;
+            delete( $self->{auth} );
+            prepare($self);
+            &do($self);    ## Need to use "&" because "do" is a perl keyword.
+        }
+        else {
+            croak("Error: $error");
+        }
+    }
+    else {
+        delete( $self->{redo} ) if $self->redo;
+        $self->{json_executed} = 1;
+        return $self->{last_response}->{'result'};
     }
-    return $self->{last_response}->{'result'};
 }
 
 sub DEMOLISH {
@@ -141,35 +198,43 @@ __END__
 Zabbix::Tiny - A small module to eliminate boilerplate overhead when using the Zabbix API
 
 =head1 SYNOPSIS
-
+  
   use strict;
   use warnings;
   use Zabbix::Tiny;
-
+  
   use Data::Dumper;
-
+  
   my $username = 'zabbix_user';
   my $password = 'secretpassword';
   my $url = 'https://zabbix.domain.com/zabbix/api_jsonrpc.php';
-
+  
   my $zabbix = Zabbix::Tiny->new(
       server   => $url,
       password => $password,
       user     => $username
   );
-
+  
   my $params = {
       output    => [qw(hostid name host)],  # Remaining paramters to 'do' are the params for the zabbix method.
       monitored => 1,
       limit     => 2,
       ## Any other params desired
   };
-
+  
+  $zabbix->prepare('host.get', $params);  # Prepare the query. 
+  print $zabbix->prepared . "\n";         # Get the JSON query without actually executing it.
+  my $host = $zabbix->do;                 # Execute the prepared query.
+  
+  # Alternately, the query can be prepared and executed in one step.
   my $hosts = $zabbix->do(
       'host.get',  # First argument is the Zabbix API method
       $params
   );
-
+  
+  # Run the same query again.  Could be useful for history and trend data
+  my $hosts = $zabbix->do;  
+  
   # Print some of the retreived information.
   for my $host (@$hosts) {
       print "Host ID: $host->{hostid} - Display Name: $host->{name}\n";
@@ -186,7 +251,12 @@ Zabbix::Tiny - A small module to eliminate boilerplate overhead when using the Z
   print "\$zabbix->post_response:\n";
   print Dumper $zabbix->post_response; # Very verbose.  Probably unnecessary.  
   
-  
+Note that as of version 1.0.6, creation of the Zabbix::Tiny object does not automatically log into the Zabbix server.
+The object will login to the Zabbix server on the first call to the C<prepare> or C<do> method.  If these methods fail 
+to connect with an invalid auth ID (for example, becasuse the user's log in timed out between the prevous call and this
+call, the module will make an attempt to log in again to get a new auth ID.  This makes the module suitable for long 
+running scripts.
+
 =head1 DESCRIPTION
 
 This module functions as a simple wrapper to eliminate boilerplate that might otherwise need to be created when interfacing with the Zabbix API.  Login to the Zabbix server is handled with the constructor. Beyond that, the primary method is the C<do> method. The user.logout method is implemented  in the object deconstructor as well, so there should be no need to explicity logout of Zabbix.
